// Weapon.java
//
// ICS 22 / CSE 22 Fall 2012
// Code Example
//
// This is the Weapon class, which is intended to specify everything that's
// common about every kind of weapon in our game, while not specifying
// anything that's specific to one particular weapon.  In this example, we
// focused on only some of the things that weapons might do in a complete
// game:
//
// * They can be fired.
// * They have an amount of ammunition, which represents how many times
//   they can be fired without being reloaded.
// * They can be reloaded with additional ammunition.
// * They sometimes are unable to fire for reasons other than not having
//   any ammunition left in them.  Different weapons are unable to fire
//   in different circumstances, but all weapons should be able to answer
//   the fundamental question "Am I able to be fired right now?"  There is
//   no "default" behavior; every kind of weapon will need to specify what
//   circumstances could cause it to be unable to fire.
//
// Each of these bullets is implemented in the class below:
//
// * There is a fire() method, which throws a CannotFireException when the
//   weapon cannot be fired -- either because it has no ammunition or because
//   something else is preventing it from being fired.
// * There is an ammunitionCount field and a getAmmunitionCount() method,
//   specifying the number of times the weapon can be fired before needing
//   to be reloaded.
// * There is a reload() method, which adds ammunition to the weapon.
// * There is a canFireWithAmmunition() method, which specifies whether the
//   weapon can fire if it has ammunition.  (Different weapons have different
//   conditions under which they can't fire, such as being jammed, broken,
//   rusted, etc.)
//
// Notice that the canFireWithAmmunition() method is different from the others.
// While it has a signature, indicating what parameters it takes (none) and
// what type of value it returns (boolean), it has no body.  So we haven't
// specified what the method will do.  There's a reason for that.  There is no
// default behavior for when methods can be fired.  All weapons will need to
// specify this differently.
//
// So, rather than put in a bogus default, we instead declare the method with
// no body, and include the word "abstract" in its signature.  An abstract
// method is a "hole" in the implementation of the Weapon class, a part
// that's intentionally missing.  We've stated that all Weapons must have a
// canFireWithAmmunition() method, but we haven't specified what that method
// will do.  We'll expect that subclasses (those that extend from Weapon) will
// fill in this abstract method with a body.
//
// Note that we've also said that the class is "abstract."  Classes with at
// least one abstract method are said to be abstract themselves.  The main
// consequence of a class being abstract is that it becomes illegal to create
// objects of the class.  So we can't have Weapon objects.  That's fine,
// though, because we only want objects of actual, concrete weapon types,
// like the TroutGun and WaterPistol classes written elsewhere in this
// example.


public abstract class Weapon
{
	private int ammunitionCount;
	
	
	public Weapon(int ammunitionCount)
	{
		this.ammunitionCount = ammunitionCount;
	}
	
	
	public int getAmmunitionCount()
	{
		return ammunitionCount;
	}
	
	
	public void reload(int additionalAmmunitionCount)
	{
		ammunitionCount += additionalAmmunitionCount;
	}
	
	
	public void fire()
	throws CannotFireException
	{
		if (ammunitionCount == 0 || !canFireWithAmmunition())
		{
			throw new CannotFireException();
		}
		else
		{
			ammunitionCount--;
		}
	}
	
	
	public abstract boolean canFireWithAmmunition();
}
