// WeaponCollection.java
//
// ICS 22 / CSE 22 Fall 2012
// Code Example
//
// This is a skeletal, incomplete class that stores a collection of weapons
// and provides the ability to fire all of them in rapid succession.
// There are a couple of interesting things going on here:
//
// (1) It's not legal to have Weapon objects (i.e., to say "new Weapon" would
//     be an error), but it is legal to have Weapon references, which is why
//     we're allowed to have an ArrayList<Weapon>.  Weapon references are
//     permitted to point to any kind of weapon, meaning, in our example, that
//     they can point to TroutGuns or WaterPistols.  Note that Weapon
//     references represent a tradeoff: they can point to any kind of weapon,
//     but you can't assume anything about what kind of weapon they point to.
//     That means, for example, that you couldn't call the clean() method on
//     the weapon, since not all weapons have a clean() method.
//
// (2) The fireAll() method makes use of a feature in Java called polymorphism.
//     In Java, whenever you call a method on an object, it's the type of the
//     object -- not the type of the reference! -- that determines which
//     version of the method gets called.  So when the fire() method is being
//     called in the fireAll() method, the version of fire() is chosen based
//     on what kind of weapons are really stored in the ArrayList.  Say, for
//     example, that the first weapon in the ArrayList is a TroutGun.  If so,
//     then TroutGun's version of fire() is called on that first weapon
//     automatically.  (Note that TroutGun simply inherits the fire() method
//     from Weapon, so it's still the Weapon.fire() code that's called.  But
//     when the fire() method calls canFire(), polymorphism again comes into
//     play; it's TroutGun's version of canFire() that's called, since the
//     object is actually a TroutGun.)
//
// One other very nice thing about this class is that we can introduce a new
// kind of weapon -- a new class extending Weapon -- without having to touch
// any of this code, since none of this code relies on the details of any
// particular weapon.

import java.util.ArrayList;


public class WeaponCollection
{
	private ArrayList<Weapon> weapons;


	public WeaponCollection(ArrayList<Weapon> weapons)
	{
		this.weapons = weapons;
	}


	public void fireAll()
	{
		for (Weapon w : weapons)
		{
			try
			{
				w.fire();
			}
			catch (CannotFireException e)
			{
			}
		}
	}
}
